Class: Cumo::NArray

Inherits:
Object
  • Object
show all
Defined in:
ext/cumo/narray/narray.c,
lib/cumo/narray/extra.rb

Overview

Cumo::NArray is the abstract super class for Numerical N-dimensional Array in the Ruby/Cumo module. Use Typed Subclasses of NArray (Cumo::DFloat, Int32, etc) to create data array instances.

Constant Summary collapse

@@warn_slow_dot =
false

Class Method Summary collapse

Instance Method Summary collapse

Class Method Details

.asarray(a) ⇒ Object



126
127
128
129
130
131
132
133
134
135
# File 'lib/cumo/narray/extra.rb', line 126

def self.asarray(a)
  case a
  when NArray
    (a.ndim == 0) ? a[:new] : a
  when Numeric, Range
    self[a]
  else
    cast(a)
  end
end

.cast(a) ⇒ Object

Convert the argument to an narray if not an narray.



110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
# File 'lib/cumo/narray/extra.rb', line 110

def self.cast(a)
  case a
  when NArray
    a
  when Array,Numeric
    NArray.array_type(a).cast(a)
  else
    if a.respond_to?(:to_a)
      a = a.to_a
      NArray.array_type(a).cast(a)
    else
      raise TypeError,"invalid type for NArray"
    end
  end
end

.column_stack(arrays) ⇒ Object

Stack 1-d arrays into columns of a 2-d array.

Examples:

x = Cumo::Int32[1,2,3]
y = Cumo::Int32[2,3,4]
Cumo::NArray.column_stack([x,y])
# => Cumo::Int32#shape=[3,2]
# [[1, 2],
#  [2, 3],
#  [3, 4]]


572
573
574
575
576
577
578
579
580
581
582
# File 'lib/cumo/narray/extra.rb', line 572

def column_stack(arrays)
  arys = arrays.map do |a|
    a = cast(a)
    case a.ndim
    when 0; a[:new, :new]
    when 1; a[true, :new]
    else; a
    end
  end
  concatenate(arys, axis:1)
end

.concatenate(arrays, axis: 0) ⇒ Object

Examples:

a = Cumo::DFloat[[1, 2], [3, 4]]
# => Cumo::DFloat#shape=[2,2]
# [[1, 2],
#  [3, 4]]

b = Cumo::DFloat[[5, 6]]
# => Cumo::DFloat#shape=[1,2]
# [[5, 6]]

Cumo::NArray.concatenate([a,b],axis:0)
# => Cumo::DFloat#shape=[3,2]
# [[1, 2],
#  [3, 4],
#  [5, 6]]

Cumo::NArray.concatenate([a,b.transpose], axis:1)
# => Cumo::DFloat#shape=[2,3]
# [[1, 2, 5],
#  [3, 4, 6]]


428
429
430
431
432
433
434
435
436
437
438
439
440
441
442
443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
475
476
477
478
479
480
# File 'lib/cumo/narray/extra.rb', line 428

def concatenate(arrays, axis:0)
  klass = (self == NArray) ? NArray.array_type(arrays) : self
  nd = 0
  arrays = arrays.map do |a|
    case a
    when NArray
      # ok
    when Numeric
      a = klass[a]
    when Array
      a = klass.cast(a)
    else
      raise TypeError, "not Cumo::NArray: #{a.inspect[0..48]}"
    end
    if a.ndim > nd
      nd = a.ndim
    end
    a
  end
  if axis < 0
    axis += nd
  end
  if axis < 0 || axis >= nd
    raise ArgumentError, "axis is out of range"
  end
  new_shape = nil
  sum_size = 0
  arrays.each do |a|
    a_shape = a.shape
    if nd != a_shape.size
      a_shape = [1] * (nd - a_shape.size) + a_shape
    end
    sum_size += a_shape.delete_at(axis)
    if new_shape
      if new_shape != a_shape
        raise ShapeError, "shape mismatch"
      end
    else
      new_shape = a_shape
    end
  end
  new_shape.insert(axis, sum_size)
  result = klass.zeros(*new_shape)
  lst = 0
  refs = [true] * nd
  arrays.each do |a|
    fst = lst
    lst = fst + (a.shape[axis - nd] || 1)
    refs[axis] = fst...lst
    result[*refs] = a
  end
  result
end

.diag_indices(m, n, k = 0) ⇒ Object

Return the k-th diagonal indices.



1062
1063
1064
1065
1066
# File 'lib/cumo/narray/extra.rb', line 1062

def self.diag_indices(m, n, k=0)
  x = Cumo::Int64.new(m, 1).seq + k
  y = Cumo::Int64.new(1, n).seq
  (x.eq y).where
end

.dstack(arrays) ⇒ Object

Stack arrays in depth wise (along third axis).

Examples:

a = Cumo::Int32[1,2,3]
b = Cumo::Int32[2,3,4]
Cumo::NArray.dstack([a,b])
# => Cumo::Int32#shape=[1,3,2]
# [[[1, 2],
#   [2, 3],
#   [3, 4]]]

a = Cumo::Int32[[1],[2],[3]]
b = Cumo::Int32[[2],[3],[4]]
Cumo::NArray.dstack([a,b])
# => Cumo::Int32#shape=[3,1,2]
# [[[1, 2]],
#  [[2, 3]],
#  [[3, 4]]]


555
556
557
558
559
560
# File 'lib/cumo/narray/extra.rb', line 555

def dstack(arrays)
  arys = arrays.map do |a|
    _atleast_3d(cast(a))
  end
  concatenate(arys, axis:2)
end

.hstack(arrays) ⇒ Object

Stack arrays horizontally (column wise).

Examples:

a = Cumo::Int32[1,2,3]
b = Cumo::Int32[2,3,4]
Cumo::NArray.hstack([a,b])
# => Cumo::Int32#shape=[6]
# [1, 2, 3, 2, 3, 4]

a = Cumo::Int32[[1],[2],[3]]
b = Cumo::Int32[[2],[3],[4]]
Cumo::NArray.hstack([a,b])
# => Cumo::Int32#shape=[3,2]
# [[1, 2],
#  [2, 3],
#  [3, 4]]


525
526
527
528
529
530
531
532
533
534
535
# File 'lib/cumo/narray/extra.rb', line 525

def hstack(arrays)
  klass = (self == NArray) ? NArray.array_type(arrays) : self
  nd = 0
  arys = arrays.map do |a|
    a = klass.cast(a)
    nd = a.ndim if a.ndim > nd
    a
  end
  dim = (nd >= 2) ? 1 : 0
  concatenate(arys, axis:dim)
end

.parse(str, split1d: /\s+/, split2d: /;?$|;/, split3d: /\s*\n(\s*\n)+/m) ⇒ Object

parse matrix like matlab, octave

Examples:

a = Cumo::DFloat.parse %[
 2 -3 5
 4 9 7
 2 -1 6
]
# => Cumo::DFloat#shape=[3,3]
# [[2, -3, 5],
#  [4, 9, 7],
#  [2, -1, 6]]


149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
# File 'lib/cumo/narray/extra.rb', line 149

def self.parse(str, split1d:/\s+/, split2d:/;?$|;/,
               split3d:/\s*\n(\s*\n)+/m)
  a = []
  str.split(split3d).each do |block|
    b = []
    #print "b"; p block
    block.split(split2d).each do |line|
      #p line
      line.strip!
      if !line.empty?
        c = []
        line.split(split1d).each do |item|
          c << eval(item.strip) if !item.empty?
        end
        b << c if !c.empty?
      end
    end
    a << b if !b.empty?
  end
  if a.size == 1
    self.cast(a[0])
  else
    self.cast(a)
  end
end

.tril_indices(m, n, k = 0) ⇒ Object

Return the indices for the lower-triangle on and below the k-th diagonal.



1046
1047
1048
1049
1050
# File 'lib/cumo/narray/extra.rb', line 1046

def self.tril_indices(m, n, k=0)
  x = Cumo::Int64.new(m, 1).seq + k
  y = Cumo::Int64.new(1, n).seq
  (x >= y).where
end

.triu_indices(m, n, k = 0) ⇒ Object

Return the indices for the uppler-triangle on and above the k-th diagonal.



1007
1008
1009
1010
1011
# File 'lib/cumo/narray/extra.rb', line 1007

def self.triu_indices(m, n, k=0)
  x = Cumo::Int64.new(m, 1).seq + k
  y = Cumo::Int64.new(1, n).seq
  (x <= y).where
end

.vstack(arrays) ⇒ Object

Stack arrays vertically (row wise).

Examples:

a = Cumo::Int32[1,2,3]
b = Cumo::Int32[2,3,4]
Cumo::NArray.vstack([a,b])
# => Cumo::Int32#shape=[2,3]
# [[1, 2, 3],
#  [2, 3, 4]]

a = Cumo::Int32[[1],[2],[3]]
b = Cumo::Int32[[2],[3],[4]]
Cumo::NArray.vstack([a,b])
# => Cumo::Int32#shape=[6,1]
# [[1],
#  [2],
#  [3],
#  [2],
#  [3],
#  [4]]


502
503
504
505
506
507
# File 'lib/cumo/narray/extra.rb', line 502

def vstack(arrays)
  arys = arrays.map do |a|
    _atleast_2d(cast(a))
  end
  concatenate(arys, axis:0)
end

Instance Method Details

#append(other, axis: nil) ⇒ Object

Append values to the end of an narray.

Examples:

a = Cumo::DFloat[1, 2, 3]
a.append([[4, 5, 6], [7, 8, 9]])
# => Cumo::DFloat#shape=[9]
# [1, 2, 3, 4, 5, 6, 7, 8, 9]

a = Cumo::DFloat[[1, 2, 3]]
a.append([[4, 5, 6], [7, 8, 9]],axis:0)
# => Cumo::DFloat#shape=[3,3]
# [[1, 2, 3],
#  [4, 5, 6],
#  [7, 8, 9]]

a = Cumo::DFloat[[1, 2, 3], [4, 5, 6]]
a.append([7, 8, 9], axis:0)
# in `append': dimension mismatch (Cumo::NArray::DimensionError)


242
243
244
245
246
247
248
249
250
251
252
253
254
255
# File 'lib/cumo/narray/extra.rb', line 242

def append(other, axis:nil)
  other = self.class.cast(other)
  if axis
    if ndim != other.ndim
      raise DimensionError, "dimension mismatch"
    end
    return concatenate(other, axis:axis)
  else
    a = self.class.zeros(size + other.size)
    a[0...size] = self[true]
    a[size..-1] = other[true]
    return a
  end
end

#concatenate(*arrays, axis: 0) ⇒ Object

Examples:

a = Cumo::DFloat[[1, 2], [3, 4]]
# => Cumo::DFloat#shape=[2,2]
# [[1, 2],
#  [3, 4]]

b = Cumo::DFloat[[5, 6]]
# => Cumo::DFloat#shape=[1,2]
# [[5, 6]]

a.concatenate(b,axis:0)
# => Cumo::DFloat#shape=[3,2]
# [[1, 2],
#  [3, 4],
#  [5, 6]]

a.concatenate(b.transpose, axis:1)
# => Cumo::DFloat#shape=[2,3]
# [[1, 2, 5],
#  [3, 4, 6]]


627
628
629
630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
656
657
658
659
660
661
662
663
664
665
666
# File 'lib/cumo/narray/extra.rb', line 627

def concatenate(*arrays, axis:0)
  axis = check_axis(axis)
  self_shape = shape
  self_shape.delete_at(axis)
  sum_size = shape[axis]
  arrays.map! do |a|
    case a
    when NArray
      # ok
    when Numeric
      a = self.class.new(1).store(a)
    when Array
      a = self.class.cast(a)
    else
      raise TypeError, "not Cumo::NArray: #{a.inspect[0..48]}"
    end
    if a.ndim > ndim
      raise ShapeError, "dimension mismatch"
    end
    a_shape = a.shape
    sum_size += a_shape.delete_at(axis - ndim) || 1
    if self_shape != a_shape
      raise ShapeError, "shape mismatch"
    end
    a
  end
  self_shape.insert(axis, sum_size)
  result = self.class.zeros(*self_shape)
  lst = shape[axis]
  refs = [true] * ndim
  refs[axis] = 0...lst
  result[*refs] = self
  arrays.each do |a|
    fst = lst
    lst = fst + (a.shape[axis - ndim] || 1)
    refs[axis] = fst...lst
    result[*refs] = a
  end
  result
end

#cov(y = nil, ddof: 1, fweights: nil, aweights: nil) ⇒ Object

under construction



1267
1268
1269
1270
1271
1272
1273
1274
1275
1276
1277
1278
1279
1280
1281
1282
1283
1284
1285
1286
1287
1288
1289
1290
1291
1292
1293
1294
1295
1296
1297
1298
1299
1300
1301
1302
1303
1304
1305
1306
1307
# File 'lib/cumo/narray/extra.rb', line 1267

def cov(y=nil, ddof:1, fweights:nil, aweights:nil)
  if y
    m = NArray.vstack([self, y])
  else
    m = self
  end
  w = nil
  if fweights
    f = fweights
    w = f
  end
  if aweights
    a = aweights
    w = w ? w * a : a
  end
  if w
    w_sum = w.sum(axis:-1, keepdims:true)
    if ddof == 0
      fact = w_sum
    elsif aweights.nil?
      fact = w_sum - ddof
    else
      wa_sum = (w * a).sum(axis:-1, keepdims:true)
      fact = w_sum - ddof * wa_sum / w_sum
    end
    if (fact <= 0).any?
      raise StandardError, "Degrees of freedom <= 0 for slice"
    end
  else
    fact = m.shape[-1] - ddof
  end
  if w
    m -= (m * w).sum(axis:-1, keepdims:true) / w_sum
    mw = m * w
  else
    m -= m.mean(axis:-1, keepdims:true)
    mw = m
  end
  mt = (m.ndim < 2) ? m : m.swapaxes(-2, -1)
  mw.dot(mt.conj) / fact
end

#deg2radObject

Convert angles from degrees to radians.



32
33
34
# File 'lib/cumo/narray/extra.rb', line 32

def deg2rad
  self * (Math::PI / 180)
end

#delete(indice, axis = nil) ⇒ Object

Examples:

a = Cumo::DFloat[[1,2,3,4], [5,6,7,8], [9,10,11,12]]
a.delete(1,0)
# => Cumo::DFloat(view)#shape=[2,4]
# [[1, 2, 3, 4],
#  [9, 10, 11, 12]]

a.delete((0..-1).step(2),1)
# => Cumo::DFloat(view)#shape=[3,2]
# [[2, 4],
#  [6, 8],
#  [10, 12]]

a.delete([1,3,5])
# => Cumo::DFloat(view)#shape=[9]
# [1, 3, 5, 7, 8, 9, 10, 11, 12]


277
278
279
280
281
282
283
284
285
286
287
288
289
# File 'lib/cumo/narray/extra.rb', line 277

def delete(indice, axis=nil)
  if axis
    bit = Bit.ones(shape[axis])
    bit[indice] = 0
    idx = [true] * ndim
    idx[axis] = bit.where
    return self[*idx].copy
  else
    bit = Bit.ones(size)
    bit[indice] = 0
    return self[bit.where].copy
  end
end

#diag(k = 0) ⇒ Object

Return a matrix whose diagonal is constructed by self along the last axis.



1069
1070
1071
1072
1073
1074
1075
# File 'lib/cumo/narray/extra.rb', line 1069

def diag(k=0)
  *shp, n = shape
  n += k.abs
  a = self.class.zeros(*shp, n, n)
  a.diagonal(k).store(self)
  a
end

#diag_indices(k = 0) ⇒ Object

Return the k-th diagonal indices.



1053
1054
1055
1056
1057
1058
1059
# File 'lib/cumo/narray/extra.rb', line 1053

def diag_indices(k=0)
  if ndim < 2
    raise NArray::ShapeError, "must be >= 2-dimensional array"
  end
  m, n = shape[-2..-1]
  NArray.diag_indices(m, n, k)
end

#diff(n = 1, axis: -1)) ⇒ Object

Calculate the n-th discrete difference along given axis.

Examples:

x = Cumo::DFloat[1, 2, 4, 7, 0]
# => Cumo::DFloat#shape=[5]
# [1, 2, 4, 7, 0]

x.diff
# => Cumo::DFloat#shape=[4]
# [1, 2, 3, -7]

x.diff(2)
# => Cumo::DFloat#shape=[3]
# [1, 1, -10]

x = Cumo::DFloat[[1, 3, 6, 10], [0, 5, 6, 8]]
# => Cumo::DFloat#shape=[2,4]
# [[1, 3, 6, 10],
#  [0, 5, 6, 8]]

x.diff
# => Cumo::DFloat#shape=[2,3]
# [[2, 3, 4],
#  [5, 1, 2]]

x.diff(axis:0)
# => Cumo::DFloat#shape=[1,4]
# [[-1, 2, 0, -2]]


947
948
949
950
951
952
953
954
955
956
957
958
959
960
961
962
963
964
965
966
967
968
969
970
971
# File 'lib/cumo/narray/extra.rb', line 947

def diff(n=1, axis:-1)
  axis = check_axis(axis)
  if n < 0 || n >= shape[axis]
    raise ShapeError, "n=#{n} is invalid for shape[#{axis}]=#{shape[axis]}"
  end
  # calculate polynomial coefficient
  c = self.class[-1, 1]
  2.upto(n) do |i|
    x = self.class.zeros(i + 1)
    x[0..-2] = c
    y = self.class.zeros(i + 1)
    y[1..-1] = c
    c = y - x
  end
  s = [true] * ndim
  s[axis] = n..-1
  result = self[*s].dup
  sum = result.inplace
  (n - 1).downto(0) do |i|
    s = [true] * ndim
    s[axis] = i..-n - 1 + i
    sum + self[*s] * c[i] # inplace addition
  end
  return result
end

#dot(b) ⇒ Cumo::NArray

Dot product of two arrays.

Parameters:

Returns:



1098
1099
1100
1101
1102
1103
1104
1105
1106
1107
1108
1109
1110
1111
1112
1113
1114
1115
1116
1117
1118
1119
1120
1121
1122
1123
1124
1125
1126
1127
1128
1129
1130
1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
1143
1144
1145
1146
1147
1148
# File 'lib/cumo/narray/extra.rb', line 1098

def dot(b)
  t = self.class::UPCAST[b.class]
  if self.ndim == 0 and b.ndim == 0
    return self * b
  end
  if [SFloat, DFloat, SComplex, DComplex].include?(t)
    b = self.class.asarray(b)
    case self.ndim
    when 1
      case b.ndim
      when 1
        self.mulsum(b, axis:-1)
      else
        self[:new, false].gemm(b).flatten
      end
    else
      case b.ndim
      when 1
        self.gemm(b[false, :new]).flatten
      else
        self.gemm(b)
      end
    end
  else
    b = self.class.asarray(b)
    case b.ndim
    when 1
      mulsum(b, axis:-1)
    else
      case ndim
      when 0
        b.mulsum(self, axis:-2)
      when 1
        self[true, :new].mulsum(b, axis:-2)
      else
        unless @@warn_slow_dot
          nx = 200
          ns = 200000
          am, an = shape[-2..-1]
          bm, bn = b.shape[-2..-1]
          if am > nx && an > nx && bm > nx && bn > nx &&
              size > ns && b.size > ns
            @@warn_slow_dot = true
            warn "\nwarning: matrix dot for #{t} is slow. Consider SFloat, DFloat, SComplex, or DComplex to use cuBLAS.\n\n"
          end
        end
        self[false, :new].mulsum(b[false, :new, true, true], axis:-2)
      end
    end
  end
end

#dsplit(indices_or_sections) ⇒ Object



773
774
775
# File 'lib/cumo/narray/extra.rb', line 773

def dsplit(indices_or_sections)
  split(indices_or_sections, axis:2)
end

#each_over_axis(axis = 0) ⇒ Object

Iterate over an axis

Examples:

> a = Cumo::DFloat.new(2,2,2).seq
> p a
Cumo::DFloat#shape=[2,2,2]
[[[0, 1],
  [2, 3]],
 [[4, 5],
  [6, 7]]]

> a.each_over_axis{|i| p i}
Cumo::DFloat(view)#shape=[2,2]
[[0, 1],
 [2, 3]]
Cumo::DFloat(view)#shape=[2,2]
[[4, 5],
 [6, 7]]

> a.each_over_axis(1){|i| p i}
Cumo::DFloat(view)#shape=[2,2]
[[0, 1],
 [4, 5]]
Cumo::DFloat(view)#shape=[2,2]
[[2, 3],
 [6, 7]]


202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
# File 'lib/cumo/narray/extra.rb', line 202

def each_over_axis(axis=0)
  unless block_given?
    return to_enum(:each_over_axis, axis)
  end
  if ndim == 0
    if axis != 0
      raise ArgumentError, "axis=#{axis} is invalid"
    end
    niter = 1
  else
    axis = check_axis(axis)
    niter = shape[axis]
  end
  idx = [true] * ndim
  niter.times do |i|
    idx[axis] = i
    yield(self[*idx])
  end
  self
end

#fliplrObject

Flip each row in the left/right direction. Same as ‘a[true, (-1..0).step(-1), …]`.



38
39
40
# File 'lib/cumo/narray/extra.rb', line 38

def fliplr
  reverse(1)
end

#flipudObject

Flip each column in the up/down direction. Same as ‘a[(-1..0).step(-1), …]`.



44
45
46
# File 'lib/cumo/narray/extra.rb', line 44

def flipud
  reverse(0)
end

#hsplit(indices_or_sections) ⇒ Object



769
770
771
# File 'lib/cumo/narray/extra.rb', line 769

def hsplit(indices_or_sections)
  split(indices_or_sections, axis:1)
end

#inner(b, axis: -1)) ⇒ Cumo::NArray

Inner product of two arrays. Same as ‘(a*b).sum(axis:-1)`.

Parameters:

  • b (Cumo::NArray)
  • axis (Integer) (defaults to: -1))

    applied axis

Returns:



1156
1157
1158
# File 'lib/cumo/narray/extra.rb', line 1156

def inner(b, axis:-1)
  mulsum(b, axis:axis)
end

#insert(indice, values, axis: nil) ⇒ Object

Insert values along the axis before the indices.

Examples:

a = Cumo::DFloat[[1, 2], [3, 4]]
a = Cumo::Int32[[1, 1], [2, 2], [3, 3]]

a.insert(1,5)
# => Cumo::Int32#shape=[7]
# [1, 5, 1, 2, 2, 3, 3]

a.insert(1, 5, axis:1)
# => Cumo::Int32#shape=[3,3]
# [[1, 5, 1],
#  [2, 5, 2],
#  [3, 5, 3]]

a.insert([1], [[11],[12],[13]], axis:1)
# => Cumo::Int32#shape=[3,3]
# [[1, 11, 1],
#  [2, 12, 2],
#  [3, 13, 3]]

a.insert(1, [11, 12, 13], axis:1)
# => Cumo::Int32#shape=[3,3]
# [[1, 11, 1],
#  [2, 12, 2],
#  [3, 13, 3]]

a.insert([1], [11, 12, 13], axis:1)
# => Cumo::Int32#shape=[3,5]
# [[1, 11, 12, 13, 1],
#  [2, 11, 12, 13, 2],
#  [3, 11, 12, 13, 3]]

b = a.flatten
# => Cumo::Int32(view)#shape=[6]
# [1, 1, 2, 2, 3, 3]

b.insert(2,[15,16])
# => Cumo::Int32#shape=[8]
# [1, 1, 15, 16, 2, 2, 3, 3]

b.insert([2,2],[15,16])
# => Cumo::Int32#shape=[8]
# [1, 1, 15, 16, 2, 2, 3, 3]

b.insert([2,1],[15,16])
# => Cumo::Int32#shape=[8]
# [1, 16, 1, 15, 2, 2, 3, 3]

b.insert([2,0,1],[15,16,17])
# => Cumo::Int32#shape=[9]
# [16, 1, 17, 1, 15, 2, 2, 3, 3]

b.insert(2..3, [15, 16])
# => Cumo::Int32#shape=[8]
# [1, 1, 15, 2, 16, 2, 3, 3]

b.insert(2, [7.13, 0.5])
# => Cumo::Int32#shape=[8]
# [1, 1, 7, 0, 2, 2, 3, 3]

x = Cumo::DFloat.new(2,4).seq
# => Cumo::DFloat#shape=[2,4]
# [[0, 1, 2, 3],
#  [4, 5, 6, 7]]

x.insert([1,3],999,axis:1)
# => Cumo::DFloat#shape=[2,6]
# [[0, 999, 1, 2, 999, 3],
#  [4, 999, 5, 6, 999, 7]]


362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
# File 'lib/cumo/narray/extra.rb', line 362

def insert(indice, values, axis:nil)
  if axis
    values = self.class.asarray(values)
    nd = values.ndim
    midx = [:new] * (ndim - nd) + [true] * nd
    case indice
    when Numeric
      midx[-nd - 1] = true
      midx[axis] = :new
    end
    values = values[*midx]
  else
    values = self.class.asarray(values).flatten
  end
  idx = Int64.asarray(indice)
  nidx = idx.size
  if nidx == 1
    nidx = values.shape[axis || 0]
    idx = idx + Int64.new(nidx).seq
  else
    sidx = idx.sort_index
    idx[sidx] += Int64.new(nidx).seq
  end
  if axis
    bit = Bit.ones(shape[axis] + nidx)
    bit[idx] = 0
    new_shape = shape
    new_shape[axis] += nidx
    a = self.class.zeros(new_shape)
    mdidx = [true] * ndim
    mdidx[axis] = bit.where
    a[*mdidx] = self
    mdidx[axis] = idx
    a[*mdidx] = values
  else
    bit = Bit.ones(size + nidx)
    bit[idx] = 0
    a = self.class.zeros(size + nidx)
    a[bit.where] = self.flatten
    a[idx] = values
  end
  return a
end

#kron(b) ⇒ Cumo::NArray

Kronecker product of two arrays.

kron(a,b)[k_0, k_1, ...] = a[i_0, i_1, ...] * b[j_0, j_1, ...]
   where:  k_n = i_n * b.shape[n] + j_n

Examples:

Cumo::DFloat[1,10,100].kron([5,6,7])
# => Cumo::DFloat#shape=[9]
# [5, 6, 7, 50, 60, 70, 500, 600, 700]

Cumo::DFloat[5,6,7].kron([1,10,100])
# => Cumo::DFloat#shape=[9]
# [5, 50, 500, 6, 60, 600, 7, 70, 700]

Cumo::DFloat.eye(2).kron(Cumo::DFloat.ones(2,2))
# => Cumo::DFloat#shape=[4,4]
# [[1, 1, 0, 0],
#  [1, 1, 0, 0],
#  [0, 0, 1, 1],
#  [0, 0, 1, 1]]

Parameters:

Returns:



1253
1254
1255
1256
1257
1258
1259
1260
1261
1262
1263
# File 'lib/cumo/narray/extra.rb', line 1253

def kron(b)
  b = NArray.cast(b)
  nda = ndim
  ndb = b.ndim
  shpa = shape
  shpb = b.shape
  adim = [:new] * (2 * [ndb - nda, 0].max) + [true, :new] * nda
  bdim = [:new] * (2 * [nda - ndb, 0].max) + [:new, true] * ndb
  shpr = (-[nda, ndb].max..-1).map { |i| (shpa[i] || 1) * (shpb[i] || 1) }
  (self[*adim] * b[*bdim]).reshape(*shpr)
end

#new_fill(value) ⇒ Object

Return an array filled with value with the same shape and type as self.



22
23
24
# File 'lib/cumo/narray/extra.rb', line 22

def new_fill(value)
  self.class.new(*shape).fill(value)
end

#new_narrayObject

Return an unallocated array with the same shape and type as self.



7
8
9
# File 'lib/cumo/narray/extra.rb', line 7

def new_narray
  self.class.new(*shape)
end

#new_onesObject

Return an array of ones with the same shape and type as self.



17
18
19
# File 'lib/cumo/narray/extra.rb', line 17

def new_ones
  self.class.ones(*shape)
end

#new_zerosObject

Return an array of zeros with the same shape and type as self.



12
13
14
# File 'lib/cumo/narray/extra.rb', line 12

def new_zeros
  self.class.zeros(*shape)
end

#outer(b, axis: nil) ⇒ Cumo::NArray

Outer product of two arrays. Same as ‘self * b`.

Examples:

a = Cumo::DFloat.ones(5)
# => Cumo::DFloat#shape=[5]
# [1, 1, 1, 1, 1]

b = Cumo::DFloat.linspace(-2,2,5)
# => Cumo::DFloat#shape=[5]
# [-2, -1, 0, 1, 2]

a.outer(b)
# => Cumo::DFloat#shape=[5,5]
# [[-2, -1, 0, 1, 2],
#  [-2, -1, 0, 1, 2],
#  [-2, -1, 0, 1, 2],
#  [-2, -1, 0, 1, 2],
#  [-2, -1, 0, 1, 2]]

Parameters:

  • b (Cumo::NArray)
  • axis (Integer) (defaults to: nil)

    applied axis (default=-1)

Returns:



1183
1184
1185
1186
1187
1188
1189
1190
1191
1192
1193
1194
1195
1196
1197
1198
1199
# File 'lib/cumo/narray/extra.rb', line 1183

def outer(b, axis:nil)
  b = NArray.cast(b)
  if axis.nil?
    self[false, :new] * ((b.ndim == 0) ? b : b[false, :new, true])
  else
    md, nd = [ndim, b.ndim].minmax
    axis = check_axis(axis) - nd
    if axis < -md
      raise ArgumentError, "axis=#{axis} is out of range"
    end
    adim = [true] * ndim
    adim[axis + ndim + 1, 0] = :new
    bdim = [true] * b.ndim
    bdim[axis + b.ndim, 0] = :new
    self[*adim] * b[*bdim]
  end
end

#percentile(q, axis: nil) ⇒ Numo::NArray

Percentile

Parameters:

  • q (Numo::NArray)
  • axis (Integer) (defaults to: nil)

    applied axis

Returns:

  • (Numo::NArray)

    return percentile

Raises:

  • (ArgumentError)


1206
1207
1208
1209
1210
1211
1212
1213
1214
1215
1216
1217
1218
1219
1220
1221
1222
1223
1224
1225
1226
1227
1228
# File 'lib/cumo/narray/extra.rb', line 1206

def percentile(q, axis: nil)
  raise ArgumentError, "q is out of range" if q < 0 || q > 100

  x = self
  unless axis
    axis = 0
    x = x.flatten
  end

  sorted = x.sort(axis: axis)
  x = q / 100.0 * (sorted.shape[axis] - 1)
  r = x % 1
  i = x.floor
  refs = [true] * sorted.ndim
  refs[axis] = i
  if i == sorted.shape[axis] - 1
    sorted[*refs]
  else
    refs_upper = refs.dup
    refs_upper[axis] = i + 1
    sorted[*refs] + r * (sorted[*refs_upper] - sorted[*refs])
  end
end

#rad2degObject

Convert angles from radians to degrees.



27
28
29
# File 'lib/cumo/narray/extra.rb', line 27

def rad2deg
  self * (180 / Math::PI)
end

#repeat(arg, axis: nil) ⇒ Object

Examples:

Cumo::NArray[3].repeat(4)
# => Cumo::Int32#shape=[4]
# [3, 3, 3, 3]

x = Cumo::NArray[[1,2],[3,4]]
# => Cumo::Int32#shape=[2,2]
# [[1, 2],
#  [3, 4]]

x.repeat(2)
# => Cumo::Int32#shape=[8]
# [1, 1, 2, 2, 3, 3, 4, 4]

x.repeat(3,axis:1)
# => Cumo::Int32#shape=[2,6]
# [[1, 1, 1, 2, 2, 2],
#  [3, 3, 3, 4, 4, 4]]

x.repeat([1,2],axis:0)
# => Cumo::Int32#shape=[3,2]
# [[1, 2],
#  [3, 4],
#  [3, 4]]


885
886
887
888
889
890
891
892
893
894
895
896
897
898
899
900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
917
# File 'lib/cumo/narray/extra.rb', line 885

def repeat(arg, axis:nil)
  case axis
  when Integer
    axis = check_axis(axis)
    c = self
  when NilClass
    c = self.flatten
    axis = 0
  else
    raise ArgumentError, "invalid axis"
  end
  case arg
  when Integer
    if !arg.kind_of?(Integer) || arg < 1
      raise ArgumentError, "argument should be positive integer"
    end
    idx = c.shape[axis].times.map { |i| [i] * arg }.flatten
  else
    arg = arg.to_a
    if arg.size != c.shape[axis]
      raise ArgumentError, "repeat size shoud be equal to size along axis"
    end
    arg.each do |i|
      if !i.kind_of?(Integer) || i < 0
        raise ArgumentError, "argument should be non-negative integer"
      end
    end
    idx = arg.each_with_index.map { |a, i| [i] * a }.flatten
  end
  ref = [true] * c.ndim
  ref[axis] = idx
  c[*ref].copy
end

#rot90(k = 1, axes = [0, 1]) ⇒ Object

Rotate in the plane specified by axes.

Examples:

a = Cumo::Int32.new(2,2).seq
# => Cumo::Int32#shape=[2,2]
# [[0, 1],
#  [2, 3]]

a.rot90
# => Cumo::Int32(view)#shape=[2,2]
# [[1, 3],
#  [0, 2]]

a.rot90(2)
# => Cumo::Int32(view)#shape=[2,2]
# [[3, 2],
#  [1, 0]]

a.rot90(3)
# => Cumo::Int32(view)#shape=[2,2]
# [[2, 0],
#  [3, 1]]


69
70
71
72
73
74
75
76
77
78
79
80
# File 'lib/cumo/narray/extra.rb', line 69

def rot90(k=1, axes=[0, 1])
  case k % 4
  when 0
    view
  when 1
    swapaxes(*axes).reverse(axes[0])
  when 2
    reverse(*axes)
  when 3
    swapaxes(*axes).reverse(axes[1])
  end
end

#split(indices_or_sections, axis: 0) ⇒ Object

Examples:

x = Cumo::DFloat.new(9).seq
# => Cumo::DFloat#shape=[9]
# [0, 1, 2, 3, 4, 5, 6, 7, 8]

x.split(3)
# => [Cumo::DFloat(view)#shape=[3]
# [0, 1, 2],
#  Cumo::DFloat(view)#shape=[3]
# [3, 4, 5],
#  Cumo::DFloat(view)#shape=[3]
# [6, 7, 8]]

x = Cumo::DFloat.new(8).seq
# => Cumo::DFloat#shape=[8]
# [0, 1, 2, 3, 4, 5, 6, 7]

x.split([3, 5, 6, 10])
# => [Cumo::DFloat(view)#shape=[3]
# [0, 1, 2],
#  Cumo::DFloat(view)#shape=[2]
# [3, 4],
#  Cumo::DFloat(view)#shape=[1]
# [5],
#  Cumo::DFloat(view)#shape=[2]
# [6, 7],
#  Cumo::DFloat(view)#shape=[0][]]


696
697
698
699
700
701
702
703
704
705
706
707
708
709
710
711
712
713
714
715
716
717
718
719
720
721
722
723
724
725
726
727
728
729
730
# File 'lib/cumo/narray/extra.rb', line 696

def split(indices_or_sections, axis:0)
  axis = check_axis(axis)
  size_axis = shape[axis]
  case indices_or_sections
  when Integer
    div_axis, mod_axis = size_axis.divmod(indices_or_sections)
    refs = [true] * ndim
    beg_idx = 0
    mod_axis.times.map do |i|
      end_idx = beg_idx + div_axis + 1
      refs[axis] = beg_idx ... end_idx
      beg_idx = end_idx
      self[*refs]
    end +
    (indices_or_sections - mod_axis).times.map do |i|
      end_idx = beg_idx + div_axis
      refs[axis] = beg_idx ... end_idx
      beg_idx = end_idx
      self[*refs]
    end
  when NArray
    split(indices_or_sections.to_a, axis:axis)
  when Array
    refs = [true] * ndim
    fst = 0
    (indices_or_sections + [size_axis]).map do |lst|
      lst = size_axis if lst > size_axis
      refs[axis] = (fst < size_axis) ? fst...lst : -1...-1
      fst = lst
      self[*refs]
    end
  else
    raise TypeError, "argument must be Integer or Array"
  end
end

#tile(*arg) ⇒ Object

Examples:

a = Cumo::NArray[0,1,2]
# => Cumo::Int32#shape=[3]
# [0, 1, 2]

a.tile(2)
# => Cumo::Int32#shape=[6]
# [0, 1, 2, 0, 1, 2]

a.tile(2,2)
# => Cumo::Int32#shape=[2,6]
# [[0, 1, 2, 0, 1, 2],
#  [0, 1, 2, 0, 1, 2]]

a.tile(2,1,2)
# => Cumo::Int32#shape=[2,1,6]
# [[[0, 1, 2, 0, 1, 2]],
#  [[0, 1, 2, 0, 1, 2]]]

b = Cumo::NArray[[1, 2], [3, 4]]
# => Cumo::Int32#shape=[2,2]
# [[1, 2],
#  [3, 4]]

b.tile(2)
# => Cumo::Int32#shape=[2,4]
# [[1, 2, 1, 2],
#  [3, 4, 3, 4]]

b.tile(2,1)
# => Cumo::Int32#shape=[4,2]
# [[1, 2],
#  [3, 4],
#  [1, 2],
#  [3, 4]]

c = Cumo::NArray[1,2,3,4]
# => Cumo::Int32#shape=[4]
# [1, 2, 3, 4]

c.tile(4,1)
# => Cumo::Int32#shape=[4,4]
# [[1, 2, 3, 4],
#  [1, 2, 3, 4],
#  [1, 2, 3, 4],
#  [1, 2, 3, 4]]


824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
843
844
845
846
847
848
849
850
851
852
853
854
855
856
857
858
# File 'lib/cumo/narray/extra.rb', line 824

def tile(*arg)
  arg.each do |i|
    if !i.kind_of?(Integer) || i < 1
      raise ArgumentError, "argument should be positive integer"
    end
  end
  ns = arg.size
  nd = self.ndim
  shp = self.shape
  new_shp = []
  src_shp = []
  res_shp = []
  (nd - ns).times do
    new_shp << 1
    new_shp << (n = shp.shift)
    src_shp << :new
    src_shp << true
    res_shp << n
  end
  (ns - nd).times do
    new_shp << (m = arg.shift)
    new_shp << 1
    src_shp << :new
    src_shp << :new
    res_shp << m
  end
  [nd, ns].min.times do
    new_shp << (m = arg.shift)
    new_shp << (n = shp.shift)
    src_shp << :new
    src_shp << true
    res_shp << n * m
  end
  self.class.new(*new_shp).store(self[*src_shp]).reshape(*res_shp)
end

#to_cObject



100
101
102
103
104
105
106
107
# File 'lib/cumo/narray/extra.rb', line 100

def to_c
  if size == 1
    Complex(self.extract_cpu)
  else
    # convert to DComplex?
    raise TypeError, "can't convert #{self.class} into Complex"
  end
end

#to_fObject



91
92
93
94
95
96
97
98
# File 'lib/cumo/narray/extra.rb', line 91

def to_f
  if size == 1
    self.extract_cpu.to_f
  else
    # convert to DFloat?
    raise TypeError, "can't convert #{self.class} into Float"
  end
end

#to_iObject



82
83
84
85
86
87
88
89
# File 'lib/cumo/narray/extra.rb', line 82

def to_i
  if size == 1
    self.extract_cpu.to_i
  else
    # convert to Int?
    raise TypeError, "can't convert #{self.class} into Integer"
  end
end

#trace(offset = nil, axis = nil, nan: false) ⇒ Object

Return the sum along diagonals of the array.

If 2-D array, computes the summation along its diagonal with the given offset, i.e., sum of ‘a`. If more than 2-D array, the diagonal is determined from the axes specified by axis argument. The default is axis=.

Parameters:

  • offset (Integer) (defaults to: nil)

    (optional, default=0) diagonal offset

  • axis (Array) (defaults to: nil)

    (optional, default=) diagonal axis

  • nan (Bool) (defaults to: false)

    (optional, default=false) nan-aware algorithm, i.e., if true then it ignores nan.



1087
1088
1089
# File 'lib/cumo/narray/extra.rb', line 1087

def trace(offset=nil, axis=nil, nan:false)
  diagonal(offset, axis).sum(nan:nan, axis:-1)
end

#tril(k = 0) ⇒ Object

Lower triangular matrix. Return a copy with the elements above the k-th diagonal filled with zero.



1015
1016
1017
# File 'lib/cumo/narray/extra.rb', line 1015

def tril(k=0)
  dup.tril!(k)
end

#tril!(k = 0) ⇒ Object

Lower triangular matrix. Fill the self elements above the k-th diagonal with zero.



1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
# File 'lib/cumo/narray/extra.rb', line 1021

def tril!(k=0)
  if ndim < 2
    raise NArray::ShapeError, "must be >= 2-dimensional array"
  end
  if contiguous?
    idx = triu_indices(k + 1)
    *shp, m, n = shape
    reshape!(*shp, m * n)
    self[false, idx] = 0
    reshape!(*shp, m, n)
  else
    store(tril(k))
  end
end

#tril_indices(k = 0) ⇒ Object

Return the indices for the lower-triangle on and below the k-th diagonal.



1037
1038
1039
1040
1041
1042
1043
# File 'lib/cumo/narray/extra.rb', line 1037

def tril_indices(k=0)
  if ndim < 2
    raise NArray::ShapeError, "must be >= 2-dimensional array"
  end
  m, n = shape[-2..-1]
  NArray.tril_indices(m, n, k)
end

#triu(k = 0) ⇒ Object

Upper triangular matrix. Return a copy with the elements below the k-th diagonal filled with zero.



976
977
978
# File 'lib/cumo/narray/extra.rb', line 976

def triu(k=0)
  dup.triu!(k)
end

#triu!(k = 0) ⇒ Object

Upper triangular matrix. Fill the self elements below the k-th diagonal with zero.



982
983
984
985
986
987
988
989
990
991
992
993
994
995
# File 'lib/cumo/narray/extra.rb', line 982

def triu!(k=0)
  if ndim < 2
    raise NArray::ShapeError, "must be >= 2-dimensional array"
  end
  if contiguous?
    *shp, m, n = shape
    idx = tril_indices(k - 1)
    reshape!(*shp, m * n)
    self[false, idx] = 0
    reshape!(*shp, m, n)
  else
    store(triu(k))
  end
end

#triu_indices(k = 0) ⇒ Object

Return the indices for the uppler-triangle on and above the k-th diagonal.



998
999
1000
1001
1002
1003
1004
# File 'lib/cumo/narray/extra.rb', line 998

def triu_indices(k=0)
  if ndim < 2
    raise NArray::ShapeError, "must be >= 2-dimensional array"
  end
  m, n = shape[-2..-1]
  NArray.triu_indices(m, n, k)
end

#vsplit(indices_or_sections) ⇒ Object

Examples:

x = Cumo::DFloat.new(4,4).seq
# => Cumo::DFloat#shape=[4,4]
# [[0, 1, 2, 3],
#  [4, 5, 6, 7],
#  [8, 9, 10, 11],
#  [12, 13, 14, 15]]

x.hsplit(2)
# => [Cumo::DFloat(view)#shape=[4,2]
# [[0, 1],
#  [4, 5],
#  [8, 9],
#  [12, 13]],
#  Cumo::DFloat(view)#shape=[4,2]
# [[2, 3],
#  [6, 7],
#  [10, 11],
#  [14, 15]]]

x.hsplit([3, 6])
# => [Cumo::DFloat(view)#shape=[4,3]
# [[0, 1, 2],
#  [4, 5, 6],
#  [8, 9, 10],
#  [12, 13, 14]],
#  Cumo::DFloat(view)#shape=[4,1]
# [[3],
#  [7],
#  [11],
#  [15]],
#  Cumo::DFloat(view)#shape=[4,0][]]


765
766
767
# File 'lib/cumo/narray/extra.rb', line 765

def vsplit(indices_or_sections)
  split(indices_or_sections, axis:0)
end